/**
 * Copyright 2008 Bluestem Software LLC.  All Rights Reserved.
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */

package org.bluestemsoftware.open.eoa.example.warehouse.manager.service;

import java.util.Date;
import java.util.HashSet;
import java.util.Set;

import org.bluestemsoftware.open.eoa.example.warehouse.manager.service.dao.CustomerOrderDAO;
import org.bluestemsoftware.open.eoa.example.warehouse.manager.service.dao.DAOException;
import org.bluestemsoftware.open.eoa.example.warehouse.manager.util.Constants;
import org.bluestemsoftware.open.eoa.example.warehouse.manager.util.WarehouseError;
import org.bluestemsoftware.open.eoa.example.warehouse.manager.vo.CustomerOrder;
import org.bluestemsoftware.open.eoa.example.warehouse.manager.vo.Item;

/**
 * Encapsulates shipping logic. Note that this service does not expose a web-service interface.
 */
public class ShippingProcessor implements Service {

    private OrderProcessor orderProcessor;
    private CustomerOrderDAO customerOrderDAO;
    private Set<Thread> sleepingThreads;

    /**
     * Calculates shipping cost.
     * 
     * @param zipCode
     * @param items
     * @return
     */
    public Double calculateShipping(String zipCode, Set<Item> items) {
        return Math.random() * 25;
    }

    /**
     * Schedules order for shipment.
     */
    public void scheduleOrder(CustomerOrder customerOrder) {

        // note that in a 'real' application, this could be a long
        // running process, i.e. it might involve human intervention
        // via a web-module interface, but for example's sake, we
        // simply re-invoke the order processor immediately

        Date estShipDate = new Date(System.currentTimeMillis() + Constants.THREE_DAYS_IN_MILLISECONDS);
        customerOrder.setEstShipDate(estShipDate);
        
        try {
            customerOrderDAO.updateCustomerOrder(customerOrder);
            orderProcessor.confirmOrder(customerOrder);
        } catch (DAOException de) {
            throw new WarehouseError(de.getMessage());
        }

        // now that order has been scheduled for fulfillment, we
        // fulfill it

        new Thread(new FulfillOrder(customerOrder)).start();

    }

    /*
     * dependency injector. set by spring framework
     */
    public void setCustomerOrderDAO(CustomerOrderDAO customerOrderDAO) {
        this.customerOrderDAO = customerOrderDAO;
    }

    /*
     * dependency injector. set by spring framework
     */
    public void setOrderProcessor(OrderProcessor orderProcessor) {
        this.orderProcessor = orderProcessor;
    }

    private class FulfillOrder implements Runnable {

        private CustomerOrder customerOrder;

        public FulfillOrder(CustomerOrder customerOrder) {
            this.customerOrder = customerOrder;
        }

        public void run() {

            // simulate time incurred while the order is actually being
            // fulfilled

            try {
                sleepingThreads.add(Thread.currentThread());
                Thread.sleep(1000 * 30);
            } catch (InterruptedException ie) {
            }

            sleepingThreads.remove(Thread.currentThread());

            // invoke complete order on our order processor
            // which notifies order management system and
            // closes out the order

            orderProcessor.completeOrder(customerOrder);

        }

    }

    /*
     * Life cycle method invoked by spring framework
     */
    public void init() {

        sleepingThreads = new HashSet<Thread>();

        // the application is 'wired' together by a third party
        // at deployment time. perform some sanity checks

        assert (customerOrderDAO != null);

    }

    /*
     * Life cycle method invoked by spring framework
     */
    public void destroy() {
        for (Thread sleepingThread : sleepingThreads) {
            sleepingThread.interrupt();
        }
    }

}
